---
{% raw -%}
env: "admin-{{ org }}"
{%- endraw %}
aws_key_name: '{{ admin_vpn_key }}'
default_ami: '{{ vpn_ami_id }}'

# VPN
openvpn_key_country: US
openvpn_key_province: NY
openvpn_key_city: New York
openvpn_key_org: KO
openvpn_key_email: admin-dev@example.net
openvpn_use_pam: no

{% raw -%}
openvpn_host: "vpn-{{ org }}.{{ canonical_zone }}"
openvpn_client_create_gateway_config: no

vpn_bucket: "{{ org }}-vpn"

openvpn_s3_conf_path: "s3://{{ vpn_bucket }}/stacks/vpn"
{%- endraw %}
# the pool of IP addresses that the VPN server manages
openvpn_server: 172.16.137.0 255.255.255.0

# the second line is the route from a VPN client to the VPC
openvpn_server_options:
  - 'up /etc/openvpn/server.up.sh'
  - 'push "route 172.20.0.0 255.255.0.0"'

openvpn_create_server_up: yes

openvpn_clients:
  - 'vpn-user1'
  - 'vpn-user2'
  - 'vpn-user3'
