[defaults]
inventory = $INFRASTRUCTURE_REPO/plugins/inventory
roles_path = $INFRASTRUCTURE_REPO/roles
filter_plugins = $INFRASTRUCTURE_REPO/plugins/filter_plugins
retry_files_save_path = ~/.ansible-retry
hash_behavior = merge

[ssh_connection]
# this needs the path defined without the use of ENV variables
ssh_args = -F __INFRA_REPO_PATH__/inventory/{{ account }}/config/private/ssh_config
