gcloud beta container \
--project {{ project }} \
clusters create {{ cluster_name }} \
{%- if cluster_ipv4_cidr %}
--cluster-ipv4-cidr={{ cluster_ipv4_cidr }} \
{%- endif %}
{%- if cluster_version %}
--cluster-version={{ cluster_version }} \
{%- endif %}
--zone={{ zone }} \
--node-locations={{ node_locations|join(',') }} \
{%- if disk_size %}
--disk-size={{ disk_size }} \
{%- endif %}
{%- if machine_type %}
--machine-type={{ machine_type }} \
{%- endif %}
{%- if network %}
--network={{ network }} \
{%- endif %}
{%- if node_labels %}
--node-labels={{ node_labels|join(',') }} \
{%- endif %}
{%- if num_nodes %}
--num-nodes={{ num_nodes }} \
{%- endif %}
{%- if image_type %}
--image-type={{ image_type }} \
{%- endif %}
{%- if username %}
--username={{ username }} \
{%- endif %}
{%- if subnetwork %}
--subnetwork={{ subnetwork }} \
{%- endif %}
{%- if scopes %}
--scopes={{ scopes|join(',') }} \
{%- endif %}
{%- if tags %}
--tags={{ tags|join('.') }}
{% endif %}
{%- if enable_autoscaling %}
--enable-autoscaling \
--min-nodes={{ min_nodes }} \
--max-nodes={{ max_nodes }} \
{%- endif %}
{#- these are enabled by default
--enable-cloud-endpoints \
--enable-cloud-logging \
--enable-cloud-monitoring \
--enable-legacy-authorization
#}
